package org.emoflon.ibex.tgg.operational.defaults.options;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.ecore.EPackage;
import org.emoflon.ibex.tgg.operational.defaults.IbexOptions;
import org.emoflon.smartemf.runtime.SmartPackage;

public class ProjectOptions extends IbexSubOptions {

	private String workspacePath;
	private String projectPath;
	private String projectName;
	private Boolean usesSmartEMF = null;

	public ProjectOptions(IbexOptions options) {
		super(options);

		projectPath = "/";
		workspacePath = "./../";
	}

	public IbexOptions workspacePath(String workspacePath) {
		this.workspacePath = workspacePath;
		return options;
	}

	public String workspacePath() {
		return workspacePath;
	}

	public IbexOptions path(String projectPath) {
		this.projectPath = projectPath;
		return options;
	}

	public String path() {
		return projectPath;
	}

	public IbexOptions name(String projectName) {
		this.projectName = projectName;
		return options;
	}

	public String name() {
		return projectName;
	}

	public boolean usesSmartEMF() {
		if (usesSmartEMF == null) {
			if (options.tgg.corrMetamodel() == null)
				return false;
			else {
				boolean isCorrSmartEMF = options.tgg.corrMetamodel() instanceof SmartPackage;
				boolean isSrcSmartEMF = options.tgg.tgg().getSrc().stream().allMatch(p -> p instanceof SmartPackage);
				boolean isTrgSmartEMF = options.tgg.tgg().getTrg().stream().allMatch(p -> p instanceof SmartPackage);
				usesSmartEMF = isCorrSmartEMF && isSrcSmartEMF && isTrgSmartEMF;

				List<EPackage> allPackages = new LinkedList<>();
				allPackages.add(options.tgg.corrMetamodel());
				allPackages.addAll(options.tgg.tgg().getSrc());
				allPackages.addAll(options.tgg.tgg().getTrg());

				if (!usesSmartEMF && !allPackages.stream().noneMatch(p -> p instanceof SmartPackage))
					throw new RuntimeException("All models have to be generated by the same model generator!");
			}
		}
		return usesSmartEMF;
	}

}
